/*******************************************************************
 * Project 2 - 2D Visualization                                    *
 * Tested in Processing 3                                          *     
 *                                                                 *
 * Author:    Susan (Su) Burtner                                   *
 * Modified:  March 4th, 2019                                      *
 *                                                                 *
 * Purpose:   Create a network where nodes represent subjects      *
 *            of maps and edges represent whether the subjects     *
 *            are in the same collection. Node size corresponds    *
 *            to number of titles in the subject.                  *
 *                                                                 *
 * Usage:     Mouse over a node to see the other titles in the     *
 *            same collection. The name of the collection will     *
 *            also appear on the left-hand side of the screen.     *
 ******************************************************************/

// Member Variables
Table data;
int rows, cols;
String[][] data_matrix;
String[] subjects;

float radiusx;
float radiusy;

ArrayList nodes;
ArrayList edges;

final float margin_to_left = 140;
final float margin_to_top = 140;

float theta;

// variables for display
int nodeX, nodeY;
color nodeColor;
boolean nodeOver = false;
int nodeSize;
int R, G, B;

//-----------------------------------------------------------------------------------------------

// Setup
void setup() {
  size(900, 900);
  
  data = loadTable("mapSubjectsTop20.csv", "header");
  
  rows = data.getRowCount();
  cols = data.getColumnCount();
  
  data_matrix = new String[rows][cols];
  
  subjects = new String[rows];
    
  nodes = new ArrayList(); 
  edges = new ArrayList();
    
  // Adding data to data_matrix
  for (int i=0; i < data_matrix.length; i++) {
    for (int j=0; j < data_matrix[0].length; j++) {
      data_matrix[i][j] = data.getString(i, j);
    }
  }
  
  // Add nodes
  radiusx = width/2-margin_to_left;
  radiusy = height/2-margin_to_top;
  
  for(int i=0; i < data_matrix.length; i++){
      nodes.add(new Node((int)(radiusx*cos(radians((360/rows)*i)-PI/2)+width/2),
        (int)(radiusy*sin(radians((360/rows)*i)-PI/2)+height/2), i,
        round(10*log(float(data_matrix[i][1]))),
        round(10*log(float(data_matrix[i][1])))));
  }
    
  // Add edges for `subjects` with shared collectionCode
  for(int i=0; i < data_matrix.length-1; i++) {
    for(int j=0; j < data_matrix.length-1; j++) {
      if(data_matrix[i][2].equals(data_matrix[j][2]) == true) {
        edges.add(new Edge((Node)nodes.get(i), (Node)nodes.get(j), data_matrix[i][2]));
      }
    }
  }
}

// Constructors
class Node {
  int xpos, ypos, id, sidex, sidey;
  
  Node(int x, int y, int i, int sx, int sy) {
    id = i;
    xpos = x;
    ypos = y;
    sidex = sx;
    sidey = sy;
  }
  
  // Get x
  int getX() {
    return(xpos);
  }
  
  // Get y
  int getY() {
    return(ypos);
  }
  
  void display(){
    smooth();
    noStroke();
    ellipse(xpos, ypos, sidex, sidey);
    fill(255, 100, 0);
  }
  
  void displayHighlight(){
    smooth();
    noStroke();
    fill(255);
    ellipse(xpos, ypos, sidex+10, sidey+10);
    fill(255, 100, 0);
    ellipse(xpos, ypos, sidex, sidey);
  }
  
  // Make a different visualization method for node selected
  void displayHighlight(int R, int G, int B){
    smooth();
    noStroke();
    fill(R, G, B);
    ellipse(xpos, ypos, sidex+10, sidey+10);
    fill(255, 100, 0);
    ellipse(xpos, ypos, sidex, sidey);
  }
}

class Edge {
  int idOr, idDest;
  String collection;
  
  Edge(Node or, Node dest, String collection){
    idOr = or.id;
    idDest = dest.id;
    this.collection = collection;
  }
  
  // Get collection
  String getColl() {
    return(collection);
  }
  
  // Get Node origin
  Node getOr() {
    Node orig = (Node) nodes.get(this.idOr);
    return(orig);
  }
  
  // Get Node destination
  Node getDest() {
    Node dest = (Node) nodes.get(this.idDest);
    return(dest);
  }
  
  // Get Node origin x
  int getOrX() {
    Node orig = (Node) nodes.get(this.idOr);
    int orX = orig.xpos;
    return(orX);
  }
  
  // Get Node origin y
  int getOrY() {
    Node orig = (Node) nodes.get(this.idOr);
    int orY = orig.ypos;
    return(orY);
  }
  
  // Get Node dest x
  int getDestX() {
    Node dest = (Node) nodes.get(this.idDest);
    int destX = dest.xpos;
    return(destX);
  }
  
  // Get Node dest y
  int getDestY() {
    Node dest = (Node) nodes.get(this.idDest);
    int destY = dest.ypos;
    return(destY);
  }
  
  void display(){
    smooth();
    Node orig = (Node) nodes.get(this.idOr);
    Node dest = (Node) nodes.get(this.idDest);
    strokeWeight(2);
    stroke(180, 244, 66, 40);
    line(orig.xpos, 
         orig.ypos, 
         dest.xpos, 
         dest.ypos);
  }
  
  void displayHighlight(){
    smooth();
    Node orig = (Node) nodes.get(this.idOr);
    Node dest = (Node) nodes.get(this.idDest);
    strokeWeight(3);
    stroke(255, 255, 255);
    line(orig.xpos, 
         orig.ypos, 
         dest.xpos, 
         dest.ypos);
  }
}

//-----------------------------------------------------------------------------------------------

// Draw
void draw(){
  background(0); 
  
  // Add title
  textSize(20);
  fill(255, 255, 255);
  textAlign(LEFT);
  text("The Top 20 Maps in the Seattle Public Library by Subject", 30, 30);
  
  textSize(14);
  text("by Susan Burtner", 30, 50);
  
  fill(255, 100, 0);
  text("The size of the nodes are proportional to the number of titles in the subject.", 30, 80);
  fill(180, 244, 66);
  text("The edges correspond to maps in the same collection.", 30, 100);
  
  fill(255, 100, 0);
  
  // Draw edges, then nodes
  for(int j=0; j < edges.size(); j++){
    Edge e = (Edge) edges.get(j);
    e.display();
  }
  
  for(int i=0; i < nodes.size(); i++) {
    Node n = (Node) nodes.get(i);
    n.display();
  }
  
  // Create highlight effect
  for(int i=0; i < nodes.size(); i++) {
    for(int j=0; j < edges.size(); j++){
      
      Node n = (Node) nodes.get(i);
      Edge e = (Edge) edges.get(j);
      
      nodeOver = overNode(n.getX(), n.getY(), n.sidex);
      
      if (nodeOver) {
        if(e.getOrX() == n.getX() && e.getOrY() == n.getY()) {
          e.displayHighlight();
          
          Node nDest = e.getDest();
          nDest.displayHighlight();
          
          textSize(14);
          fill(255, 255, 255);
          textAlign(LEFT);
          text("Current map collection:", 30, 180);
          textSize(16);
          fill(255, 255, 0);
          text(e.getColl(), 30, 200);
        }
        n.displayHighlight(255, 255, 0);
      }
    }
  }
  
  // Add the subjects
  drawSubjects();
 
}

//-----------------------------------------------------------------------------------------------

void drawSubjects() {
  textSize(12);
  fill(255, 255, 255);
  
  pushMatrix();
  
  // Align text according to where it falls on circle, adding the radius of its circle
  // Text on the top right half
  for(int i=0; i < data_matrix.length/4; i++){
    text(data_matrix[i][0], radiusx*cos(radians((360/rows)*i)-PI/2)+
        width/2+round(10*log(float(data_matrix[i][1]))),
      radiusy*sin(radians((360/rows)*i)-PI/2)+
        height/2-round(10*log(float(data_matrix[i][1]))), 80, 80);
    textAlign(LEFT);
  }
  
  // Text on the bottom right half
  for(int i=data_matrix.length/4; i < 2*data_matrix.length/4; i++){
    text(data_matrix[i][0], radiusx*cos(radians((360/rows)*i)-PI/2)+
        width/2+round(10*log(float(data_matrix[i][1]))),
      radiusy*sin(radians((360/rows)*i)-PI/2)+
        height/2-round(10*log(float(data_matrix[i][1]))), 80, 80);
    textAlign(LEFT);
  }
  
  // Text on the bottom left half
  for(int i=2*data_matrix.length/4; i < 3*data_matrix.length/4; i++){
    text(data_matrix[i][0], radiusx*cos(radians((360/rows)*i)-PI/2)+
        width/2-round(10*log(float(data_matrix[i][1]))),
      radiusy*sin(radians((360/rows)*i)-PI/2)+
        height/2-round(10*log(float(data_matrix[i][1]))), -80, 80);
    textAlign(RIGHT);
  }
  
  // Text on the top left half
  for(int i=3*data_matrix.length/4; i < data_matrix.length; i++){
    text(data_matrix[i][0], radiusx*cos(radians((360/rows)*i)-PI/2)+
        width/2-round(10*log(float(data_matrix[i][1]))),
      radiusy*sin(radians((360/rows)*i)-PI/2)+
        height/2-round(10*log(float(data_matrix[i][1]))), -80, 80);
    textAlign(RIGHT);
  }
  
  popMatrix();
}

// Method for mouse over, taken from Processing references, with modifications:
// https://processing.org/examples/rollover.html

boolean overNode(int x, int y, int diameter) {
  float disX = x - mouseX;
  float disY = y - mouseY;
  if(sqrt(sq(disX) + sq(disY)) < diameter/2 ) {
    return true;
  } else {
    return false;
  }
}
